"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var ImageType;
(function (ImageType) {
    ImageType["JPEG"] = "image/jpeg";
    ImageType["PNG"] = "image/png";
})(ImageType = exports.ImageType || (exports.ImageType = {}));
var SupportedImageMetaTag;
(function (SupportedImageMetaTag) {
    SupportedImageMetaTag["XResolution"] = "XResolution";
    SupportedImageMetaTag["YResolution"] = "YResolution";
    SupportedImageMetaTag["Orientation"] = "Orientation";
})(SupportedImageMetaTag = exports.SupportedImageMetaTag || (exports.SupportedImageMetaTag = {}));
exports.ExifOrientation = {
    'top-left': 1,
    'top-right': 2,
    'bottom-right': 3,
    'bottom-left': 4,
    'left-top': 5,
    'right-top': 6,
    'right-bottom': 7,
    'left-bottom': 8,
};
var ORIENT_TRANSFORMS = {
    1: 'none',
    2: 'rotateY(180deg)',
    3: 'rotate(180deg)',
    4: 'rotate(180deg) rotateY(180deg)',
    5: 'rotate(270deg) rotateY(180deg)',
    6: 'rotate(90deg)',
    7: 'rotate(90deg) rotateY(180deg)',
    8: 'rotate(270deg)',
};
var Orientation = SupportedImageMetaTag.Orientation, XResolution = SupportedImageMetaTag.XResolution, YResolution = SupportedImageMetaTag.YResolution;
var pngChunksExtract;
var loadImage;
function fileToArrayBuffer(file) {
    return new Promise(function (resolve, reject) {
        var reader = new FileReader();
        reader.addEventListener('load', function () {
            var array = new Uint8Array(reader.result);
            resolve(array);
        });
        reader.addEventListener('error', reject);
        reader.readAsArrayBuffer(file);
    });
}
function readJPEGExifMetaData(file) {
    var _this = this;
    return new Promise(function (resolve, reject) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
        var module_1;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!!loadImage) return [3, 2];
                    return [4, Promise.resolve().then(function () { return require('blueimp-load-image'); })];
                case 1:
                    module_1 = _a.sent();
                    loadImage = module_1.default || module_1;
                    _a.label = 2;
                case 2:
                    loadImage.parseMetaData(file, function (data) {
                        try {
                            var tags_1 = data && data.exif ? data.exif.getAll() : {};
                            Object.keys(tags_1).forEach(function (key) {
                                var value = tags_1[key];
                                if (typeof value === 'object' &&
                                    (key === XResolution || key === YResolution) &&
                                    'numerator' in value) {
                                    tags_1[key] = value.numerator;
                                }
                                if (typeof tags_1[key] === 'number') {
                                    tags_1[key] = "" + tags_1[key];
                                }
                            });
                            resolve(tags_1);
                        }
                        catch (e) {
                            reject(e);
                        }
                    });
                    return [2];
            }
        });
    }); });
}
function readPNGXMPMetaData(file) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var module_2, buffer, chunks;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!!pngChunksExtract) return [3, 2];
                    return [4, Promise.resolve().then(function () { return require('png-chunks-extract'); })];
                case 1:
                    module_2 = _a.sent();
                    pngChunksExtract = module_2.default || module_2;
                    _a.label = 2;
                case 2: return [4, fileToArrayBuffer(file)];
                case 3:
                    buffer = _a.sent();
                    chunks = pngChunksExtract(buffer);
                    return [4, parsePNGChunks(chunks)];
                case 4: return [2, _a.sent()];
            }
        });
    });
}
function parsePNGChunks(chunks) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var iTXt, pHYs, i, chunk, dv, unitSpecifier, PixelPerUnitX, PixelPerUnitY;
        return tslib_1.__generator(this, function (_a) {
            iTXt = '';
            pHYs = {};
            for (i = 0; i < chunks.length; ++i) {
                chunk = chunks[i];
                if (chunk.name === 'IEND') {
                    break;
                }
                if (chunk.name === 'iTXt') {
                    iTXt = String.fromCharCode.apply(null, chunk.data);
                }
                if (chunk.name === 'pHYs') {
                    dv = new DataView(chunk.data.buffer);
                    unitSpecifier = dv.getUint8(8);
                    if (unitSpecifier === 1) {
                        PixelPerUnitX = dv.getUint32(0);
                        PixelPerUnitY = dv.getUint32(4);
                        pHYs = { PixelPerUnitX: PixelPerUnitX, PixelPerUnitY: PixelPerUnitY };
                    }
                }
            }
            return [2, { iTXt: iTXt, pHYs: pHYs }];
        });
    });
}
function parseXMPMetaData(xmpMetaData) {
    var metadata = {};
    var tags = xmpMetaData.match(/<(tiff|exif):.+>/gi);
    if (tags) {
        tags.forEach(function (tag) {
            var match = tag.match(/<(tiff|exif):([^>]+)>([^<]+)/i);
            if (match) {
                var name_1 = match[2];
                metadata[name_1] = match[3];
            }
        });
    }
    return metadata;
}
function readImageMetaTags(file) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var type, _a, iTXt, pHYs, xmpMetaData, e_1;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    type = file.type;
                    _b.label = 1;
                case 1:
                    _b.trys.push([1, 6, , 7]);
                    if (!(type === ImageType.PNG)) return [3, 3];
                    return [4, readPNGXMPMetaData(file)];
                case 2:
                    _a = _b.sent(), iTXt = _a.iTXt, pHYs = _a.pHYs;
                    xmpMetaData = tslib_1.__assign({}, parseXMPMetaData(iTXt), pHYs);
                    return [2, xmpMetaData];
                case 3:
                    if (!(file.type === ImageType.JPEG)) return [3, 5];
                    return [4, readJPEGExifMetaData(file)];
                case 4: return [2, _b.sent()];
                case 5: return [3, 7];
                case 6:
                    e_1 = _b.sent();
                    return [3, 7];
                case 7: return [2, null];
            }
        });
    });
}
exports.getCssFromImageOrientation = function (orientation) {
    return ORIENT_TRANSFORMS[orientation];
};
function getOrientation(file) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var tags, tagValue, numericValue;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, readImageMetaTags(file)];
                case 1:
                    tags = _a.sent();
                    if (tags && tags[Orientation]) {
                        tagValue = tags[Orientation];
                        if (tagValue) {
                            numericValue = parseInt(tagValue, 10);
                            if (isNaN(numericValue)) {
                                return [2, exports.ExifOrientation[tagValue]];
                            }
                            return [2, numericValue];
                        }
                    }
                    return [2, 1];
            }
        });
    });
}
exports.getOrientation = getOrientation;
